package myLibrary;

import java.io.*;
import java.util.*;
import alice.tuprolog.*;

import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;
import com.sun.xml.parser.Resolver;


/**
 * SAXParser che permette di convertire un xml in
 * una struttura dati compatibile con l'engine tuProlog.
 *
 * @author Alex Benini
 */
public class SAXParser extends HandlerBase
{

	private Term  doc;
	private Term  local   = null;
	private Stack stack   = new Stack();
	private Stack stkAttr = new Stack();
	private boolean flag  = false;
	private int   typeParsing = 0;
	// TypeParsing //
	/* All tree    = 0
	 * Elem & Attr = 1
	 * Only Elem   = 2
	 */


	/**
	*
	*/
/*	public Term parseAll(String file)  throws Exception  {
		typeParsing = 2; //All tree
		return parse(file);
	}
*/
	/**
	* Esegue il parsing considerando gli elementi e i loro attributi e omettendo i caratteri.
	* @param file File da processare
	*/
	public Term parseElemAttr(String file)  throws Exception  {
		typeParsing = 1; //Elem & Attr
		return parse(file);
	}

	/**
	* Esegue il parsing considerando gli elementi e omettendo attributi e caratteri.
	* @param file File da processare
	*/
	public Term parseElem(String file)  throws Exception  {
		typeParsing = 0; //Only Elem
		return parse(file);
	}


	//Istanzia le classi necessarie per effettuare il parsing vero e proprio
	private Term parse(String file) throws Exception {

  		  InputSource input;

			//Turn the filename into an XML input source
			input = Resolver.getSource(new File(file));

			//Get an instance of the non-validating parser.
			Parser parser;
			parser = ParserFactory.makeParser("com.sun.xml.parser.Parser");
			parser.setDocumentHandler(this);

			//Parse the input
			parser.parse(input);

		return doc;
	}



	//=================================================
	// SAX DocumentHandler methods
	//=================================================

	public void startDocument() throws SAXException {}


	public void endDocument() throws SAXException {
		doc = local;
	}


	public void startElement(String name, AttributeList attrs) throws SAXException
	{
		try {
			int attlen = attrs.getLength();
			Term[] attarg = new Term[attlen];
			for (int i=0; i<attlen; i++){
			    attarg[i] = new Struct((String)attrs.getValue(i));
			}//for
			//Salvo gli attributi nello stack stkAttr
			stkAttr.push(attarg);
			//Salvo nello stack l'ultima struttura dati ad albero istanziata
			stack.push(local);
			local = null;
		} catch(Exception e) {
			System.out.println("Error in startElement");
			e.printStackTrace();
		}
		flag = false; //Significa che sono appena sceso in un nodo del documento
	}


	public void endElement(String name) throws SAXException
	{
		switch (typeParsing) {
			case 0 ://Parsing dei soli elementi
				try {
					/* se ho appena processato un sottoalbero allora lo 
					 * converto in una struttura dati Struct 
					 */
					if (flag) { 
						Vector elems = new Vector();
						while (local!=null) {
							elems.addElement(local);
							local = (Term)stack.pop();
						}//while
						elems.addElement(new Struct("elem"));//indica il tipo di albero
						stkAttr.pop();
						if (elems.size()==0) {
							local = new Struct(name,new Struct("elem"));
						  } else {
							int len = elems.size();
							Term[] arg = new Term[len];
							for (int i=0; i<len; i++){
							    arg[i] = (Term)elems.elementAt(i);
							}//for
							//salva la struttura dati appena creata
							local = new Struct(name,arg);
						}//if
					}//then_flag
					  else {
						local = new Struct(name,new Struct("elem"));
					}//if_flag
				} catch(Exception e) {
					System.out.println("Error in endElement sezione typeParsing=0");
					e.printStackTrace(System.out);
				}
				flag = true;//Significa che sono appena salito da un nodo del documento
				break;
			case 1 ://Parsing degli attributi+elementi
				try {
					if (flag) {
						Vector elems = new Vector();
						while (local!=null) {
							elems.addElement(local);
							local = (Term)stack.pop();
						}//while

						Term[] attarg = (Term[])stkAttr.pop();

						elems.addElement(new Struct(attarg));
						elems.addElement(new Struct("attr"));
						if (elems.size()==0) {
							local = new Struct(name,new Struct(attarg),new Struct("attr"));
						  } else {
							int len = elems.size();
							Term[] arg = new Term[len];
							for (int i=0; i<len; i++){
							    arg[i] = (Term)elems.elementAt(i);
							}//for
							local = new Struct(name,arg);
						}//if
					}//then_flag
					  else {
						Term[] attarg = (Term[])stkAttr.pop();
						local = new Struct(name,new Struct(attarg),new Struct("attr"));
					}//if_flag
				} catch(Exception e) {
					System.out.println("Error in endElement sezione typeParsing=1");
					e.printStackTrace(System.out);
				}
				flag = true;//Significa che sono appena salito da un nodo del documento
				break;
			case 2 :
				break;
		}//switch
		System.out.println(local);
	}


	public void characters(char buf[], int offset, int len) throws SAXException
	{
	}


}//fine